/*
 *  SynDetector
 *
 *  MSC in Telecommunications
 *  Denial of Service Study
 *	Queens University Belfast
 *
 *  Author   : Matthew Hutchinson
 *  Contact  : m.hutchinson@ijug.org
 *  Web      : http://hutchinson.ijug.org
 *
 *  Created  : 01-Aug-03
 *  Filename : SynDetectorDlg.cpp
 *             Main dialog window construction and event handlers
 *
 *  Library     : TCPIP_LIBv4 by Komodia Inc.  <open source>
 *	Download    : http://www.komodia.com/tcpip_lib4.zip
 *  API Support : http://www.komodia.com/help/default.htm 
 *  Copyright   : Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *                All rights reserved.
 *
 *  NOTICE      : This is a Proof Of Concept program for educational purposes only.
 *
 */


// for precompiled header directive
#include "stdafx.h"
// custom header files
#include "SynDetector.h"
#include "SynDetectorDlg.h"
#include "ClientSocket.h"
// TCPIP_LIBv4 library
#include "Interfaces.h"


// CAboutDlg dialog used for App About
class CAboutDlg : public CDialog
{
	public: CAboutDlg();

	// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

	// Implementation
	protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}


void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



// CSynDetectorDlg dialog
CSynDetectorDlg::CSynDetectorDlg(CWnd* pParent)	: CDialog(CSynDetectorDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSynDetectorDlg)
	m_Sniff = FALSE;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}


void CSynDetectorDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSynDetectorDlg)
	DDX_Control(pDX, IDC_DATALIST, m_DataList);
	DDX_Control(pDX, IDC_INTERFACELIST, m_InterfaceList);
	DDX_Control(pDX, IDC_EDIT1, m_FloodCount);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSynDetectorDlg, CDialog)
	//{{AFX_MSG_MAP(CSynDetectorDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_DESTROY()
	ON_BN_CLICKED(ID_SNIFF, OnSniff)
	ON_BN_CLICKED(ID_CLEAR, OnClear)
	ON_BN_CLICKED(ID_ABOUT, OnAbout)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



/*
 *  BOOL OnInitDialog()
 *  Initialise the main SynDetector dialog
 *
*/
BOOL CSynDetectorDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu
	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// set the icon for this dialog.
	SetIcon(m_hIcon, TRUE);

	// initialise TCPIP_LIBv4 library for socket communication
	if (CSpoofBase::InitializeSockets())
	{
		// create a new custom Client Socket
		m_Socket=new CClientSocket(&m_DataList, &m_FloodCount);
		m_Socket->Create();
	}
	else
		return FALSE;

	// build socket interface list
	if(!BuildInterfaceList())
	{
		AfxMessageBox("Cannot find any network interfaces ...", MB_ICONERROR);
		return FALSE;
	}

	// start sniffing on default interface
	OnSniff();
	return TRUE;
}


/*
 *  OnSysCommand(UINT, LPARAM)
 *  Show about dialog when About menu item is selected
 *
*/
void CSynDetectorDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}


/*
 *  OnPaint()
 *  generate icon for minimize button
 *
*/
void CSynDetectorDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this);
		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}


/*
 *  HCURSOR OnQueryDragIcon()
 *  show cursor during minimised drag
 *
*/
HCURSOR CSynDetectorDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


/*
 *  OnDestroy() 
 *  destroy dialog resources used
 *
*/
void CSynDetectorDlg::OnDestroy() 
{
	CDialog::OnDestroy();
	
	// Delete the socket
	delete m_Socket;
	// Close all open or active sockets
	CSpoofBase::ShutdownSockets();
}


/*
 *  BOOL BuildInterfaceList()
 *  Use CInterfaces class to get a list of network interfaces available
 *  Add each to the Interface List Box, select the last one and return
 *
*/
BOOL CSynDetectorDlg::BuildInterfaceList()
{
	// get the list of interfaces
	CInterfaces* pInter;
	pInter = new CInterfaces;

	// only if we have the interfaces
	if (pInter->GetInterfaces())
	{
		// build the list
		BOOL bQuit;
		bQuit=FALSE;

		while (!bQuit)
		{
			// get the interface address
			LPSTR lpInterface;
			lpInterface=pInter->LongToString(pInter->GetAddress());
	
			// add it to the list
			m_InterfaceList.AddString(lpInterface);

			// get next interface
			bQuit=!pInter->MoveNext();
		}

		// select a default interface
		m_InterfaceList.SetCurSel(m_InterfaceList.GetCount()-1);
		delete pInter;		
		return TRUE;
	}

	delete pInter;
	return FALSE;
}


/*
 *  OnSniff()
 *  When 'Select' button is pressed
 *  Bind the socket to the selected interface, 
 *  reset the flood count and start sniffing packets. 
 *
*/
void CSynDetectorDlg::OnSniff() 
{
	CString strInterface;
	m_InterfaceList.GetText(m_InterfaceList.GetCurSel(),strInterface);
	
	// bind to an interface
	if (m_Socket->Bind((LPCSTR)strInterface))
	{
		m_Socket->ResetFloodCount();
		// start sniffing packets
		if (!m_Socket->Sniff(TRUE))
			AfxMessageBox("Cannot monitor packets on this interface !", MB_ICONERROR);
	}
}


/*
 *  OnClear()
 *  When 'Clear Data' button is pressed
 *  Reset the SYN flood count and clear the data window 
 *
*/
void CSynDetectorDlg::OnClear() 
{	
	m_Socket->ResetFloodCount();
	// clear the contents of the TCP data window
	if(m_DataList.GetCount() != 0)
	{
		m_DataList.ResetContent();
		AfxMessageBox("Cleared the capture window ...", MB_ICONINFORMATION);
	}
}


/*
 *  OnAbout()
 *  When 'About' button is pressed
 *  Show the About dialog 
 *
*/
void CSynDetectorDlg::OnAbout() 
{	
	CAboutDlg dlgAbout;
	dlgAbout.DoModal();
}
